// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GMAIL_ACCOUNT_STATE_CONNECTING = "connecting";
const GMAIL_ACCOUNT_STATE_LOGGED_OUT = "logged-out";
const GMAIL_ACCOUNT_STATE_LOGGED_IN = "logged-in";
const GMAIL_ACCOUNT_STATE_NEW_MAIL = "new-mail";
const GMAIL_ACCOUNT_STATE_NO_PASSWORD = "no-password";
const GMAIL_ACCOUNT_STATE_ERROR_PASSWORD = "error-password";
const GMAIL_ACCOUNT_STATE_ERROR_CAPTCHA = "error-captcha";
const GMAIL_ACCOUNT_STATE_ERROR_NETWORK = "error-network";
const GMAIL_ACCOUNT_STATE_ERROR_SERVER = "error-server";
const GMAIL_ACCOUNT_STATE_ERROR_TIMEOUT = "error-timeout";

const GMAIL_SCHEME = Array("http", "https");

function onOverlayUnload() {
  try {
    gOverlay._prefWatch.removeObserver("", gOverlay);
    gOverlay._observer.removeObserver(gOverlay, "account-login");
    gOverlay._observer.removeObserver(gOverlay, "reset-unread-count");
  } catch(e) {}
  
  for (var i = 0; i < gOverlay._manager.getNumAccounts(); i++) {
    var account = gOverlay._manager.getAccount(i);
    try {
      gOverlay._observer.removeObserver(gOverlay, account.username);
    } catch(e) {}
  }
}

var gOverlay = {
  _bundle: null,
  _manager: null,
  _branch: null,
  _observer: null,
  _current: -1,
  _prefWatch: null,
  _loadMail: false,
  _loadMailStream: null,
  _scheme: "https",
  
  load: function()
  {
    this._bundle = document.getElementById("gm-overlay-bundle");
    
    // Loads the account manager
    this._manager = Components.classes["@longfocus.com/gmanager/manager;1"].getService(Components.interfaces.nsIGmailManager);
    this._manager.load();
    
    // Adds the preferences observer for the "gmanager" branch
    this._branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(GMANAGER_BRANCH);
    this._prefWatch = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    this._prefWatch.addObserver("", this, false);
    
    // Adds the observers
    this._observer = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this._observer.addObserver(this, "account-login", false);
    this._observer.addObserver(this, "reset-unread-count", false);
    
    for (var i = 0; i < this._manager.getNumAccounts(); i++) {
      var account = this._manager.getAccount(i);
      try {
        this._observer.addObserver(this, account.username, false);
      } catch(e) {}
    }
    
    // Checks for version
    if (this._branch.getCharPref("version") != getVersion())
      setTimeout("gOverlay._welcome();", 10); // Welcome stuff
    
    // Startup stuff
    this._startup();
  },
  
  _startup: function()
  {
    // Position the statusbar
    this._checkStatusbar();
    
    // Sets the type of connection (secured or not)
    var secure = this._branch.getBoolPref("secured-con");
    this._scheme = GMAIL_SCHEME[(secure ? 1 : 0)];
    
    // Get the current value stored
    var current = this._branch.getCharPref("current").split(",")[0];
    var index = this._manager.getAccountIndex(current);
    
    // Select the current account
    this.selectAccount(index, false);
    
    // Automatically login accounts if necessary
    if (this._branch.getBoolPref("auto-login"))
      this.select("gm-overlay-menuitem-loginall");
  },
  
  _welcome: function()
  {
    // Checks if this is the first time running
    if (this._branch.getBoolPref("first-time"))
    {
      // Opens the imports window
      window.openDialog("chrome://gmanager/content/imports.xul", "imports", "centerscreen,chrome,resizable=yes", this);
      
      // Sets the first-time preference to false
      this._branch.setBoolPref("first-time", false);
    }
    
    /*
    // Opens the welcome window
    window.openDialog("chrome://gmanager/content/welcome.xul", "welcome", "modal,centerscreen,chrome,resizable=no", this);
    */
    
    // Sets the version preference to the current version
    this._branch.setCharPref("version", getVersion());
  },
  
  select: function(aId)
  {
    switch (aId) {
      case "gm-overlay-menuitem-loginall":
      {
        // Loop through all of the accounts
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          
          // Check if account is logged in
          if (!account.loggedIn)
            this.doAccountLogin(account); // Login account
        }
        break;
      }
      case "gm-overlay-menuitem-logoutall":
      {
        // Loop through all of the accounts
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          account.logout(); // Logout account
        }
        break;
      }
      case "gm-overlay-menuitem-all-checknow":
      {
        // Loop through all of the accounts
        for (var i = 0; i < this._manager.getNumAccounts(); i++) {
          var account = this._manager.getAccount(i);
          
          // Check if account is logged in
          if (account.loggedIn)
            account.check(); // Check account
        }
        break;
      }
      case "gm-overlay-menuitem-loginout":
      {
        var account = this._manager.getAccount(this._current);
        account.loggedIn ? account.logout() : this.doAccountLogin(account);
        break;
      }
      case "gm-overlay-menuitem-checknow":
      {
        var account = this._manager.getAccount(this._current);
        account.check(); // Check account
        break;
      }
      case "gm-overlay-menuitem-showsnippets":
      {
        var account = this._manager.getAccount(this._current);
        var link = this._branch.getBoolPref("system-notify-clickable");
        this._addAccountSnippets(account, link); // Show snippets
        break;
      }
    }
  },
  
  selectAccount: function(aNum, aLogin)
  {
    this._current = aNum;    
    
    var account = this._manager.getAccount(this._current);
    
    if (!account)
      return;
    
    if (aLogin)
      account.loggedIn ? account.check() : this.doAccountLogin(account);
    
    // Toolbar
    if (document.getElementById("gm-toolbar-button")) {
      document.getElementById("gm-toolbar-button").setAttribute("status", account.status);
      document.getElementById("gm-toolbar-button").setAttribute("newMail", this._getUnread(account) > 0);
      document.getElementById("gm-toolbar-button").setAttribute("unread", account.loggedIn ? this._getUnread(account) : "");
      document.getElementById("gm-toolbar-button").setAttribute("username", !this._branch.getBoolPref("hide-username") ? account.username : "");
    }
    
    // Statusbar
    document.getElementById("gm-statusbar-panel").setAttribute("status", account.status);
    document.getElementById("gm-statusbar-panel").setAttribute("newMail", this._getUnread(account) > 0);
    document.getElementById("gm-statusbar-panel").setAttribute("unread", account.loggedIn ? this._getUnread(account) : "");
    document.getElementById("gm-statusbar-panel").setAttribute("username", !this._branch.getBoolPref("hide-username") ? account.username : "");
    
    this._branch.setCharPref("current", account.username + "," + this._current);
  },
  
  doAccountLogin: function(aAccount)
  {
    var check = this._observer.enumerateObservers(aAccount.username);
    
    if (!check.hasMoreElements()) {
      try {
        this._observer.addObserver(this, aAccount.username, false);
      } catch(e) {}
    }
    
    // Login account
    aAccount.login();
  },
  
  loadGmail: function(aEvent, aAccount)
  {
    if (aEvent && (aEvent.button == 2))
      return;
    
    if (aAccount != null || this._current > -1)
    {
      var account = (aAccount ? aAccount : this._manager.getAccount(this._current));
      
      if (account == null) {
        window.openDialog('chrome://gmanager/content/preferences.xul', null, 'chrome, dependent=yes', 'Add');
        return;
      }
      
      if (!account.loggedIn) {
        this.doAccountLogin(account);
      }
      else
      {
        // Resets unread count appropriately
        if (this._branch.getBoolPref("unread-reset"))
          this._observer.notifyObservers(null, "reset-unread-count", null);
        
        // Setup data to be sent
        var data = "ltmpl=wsad&ltmplcache=2&continue=" + this._scheme + "%3A%2F%2Fmail.google.com%2Fmail%2F%3F&service=mail&rm=false&ltmpl=wsad&Email=" + 
                   encodeURIComponent(account.username) + "&Passwd=" + encodeURIComponent(account.password) + "&null=Sign+in";
        
        // Check for middle mouse click
        var loc = (aEvent.button == 1) ? 2 : this._branch.getIntPref("gmailweb-loc");
        
        // Loads the URI appropriately
        gURI.load(data, loc, false);
      }
    }
    else {
      window.openDialog('chrome://gmanager/content/preferences.xul', null, 'chrome, dependent=yes', 'Add');
    }
  },
  
  loadGmailCompose: function(aUsername, aCompose, aMiddle)
  {
    var index = this._manager.getAccountIndex(aUsername);
    var account = this._manager.getAccount(index);
    
    // Setup data to be sent
    var data = "ltmpl=wsad&ltmplcache=2&continue=" +
               this._scheme + "%3A%2F%2Fmail.google.com%2Fmail%2F%3Fview%3Dcm%26fs%3D1%26" +
               "to%3D" + aCompose.to + 
               "%26su%3D" + aCompose.su + 
               "%26body%3D" + aCompose.body + 
               "%26cc%3D" +aCompose.cc + 
               "%26bcc%3D" + aCompose.bcc + 
               "&service=mail&rm=false&ltmpl=wsad&" + 
               "Email=" + encodeURIComponent(account.username) + 
               "&Passwd=" + encodeURIComponent(account.password) + "&null=Sign+in";
    
    // Location of the tab to open
    var loc = (aMiddle) ? 2 : this._branch.getIntPref("compose.mailto.loc");
    
    // Loads the URI appropriately
    gURI.load(data, loc, true);
  },
  
  _checkStatusbar: function()
  {
    var sbar = getEBI("status-bar");
    var sbarPanel = getEBI("gm-statusbar-panel");
    
    sbarPanel.hidden = !this._branch.getBoolPref("statusbar");
    sbar.removeChild(sbarPanel);
    
    if (this._branch.getBoolPref("statusbar.always-last"))
      sbar.appendChild(sbarPanel);
    else {
      var pos = this._branch.getIntPref("statusbar.position");
      sbar.insertBefore(sbarPanel, sbar.childNodes[pos]);
    }
  },
  
  _getUnread: function(aAccount)
  {
    var unreadCount = 0;
    
    if (this._branch.getBoolPref("unread-count-inbox"))
      unreadCount += aAccount.inboxUnread;
     
    if (this._branch.getBoolPref("unread-count-spam"))
      unreadCount += aAccount.spamUnread;
    
    if (this._branch.getBoolPref("unread-count-labels")) {
      for (var i = 0; i < aAccount.getNumLabels(); i++) {
        var label = aAccount.getLabel(i).split(",");
        unreadCount += parseInt(label[1]);
      }
      
      if (this._branch.getBoolPref("unread-count-inbox"))
        for (var i = 0; i < aAccount.getNumSnippets(); i++)
          unreadCount -= aAccount.getSnippetNumLabels(i);
    }
    
    if (!this._branch.getBoolPref("unread-count-inbox") &&
        !this._branch.getBoolPref("unread-count-spam") &&
        !this._branch.getBoolPref("unread-count-labels")) unreadCount = "";
    
    return unreadCount;
  },
  
  _addAccountSnippets: function(aAccount, aLink)
  {
    var conv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    conv.charset = "UTF-8";
    
    for (var i = 0; i < aAccount.getNumSnippets(); i++)
    {
      var subj = aAccount.username + " - " + (i + 1) + " " + 
             this._bundle.getString("of") + " " + 
             aAccount.getNumSnippets() + " - " + 
             aAccount.getSnippetTime(i) + " - " + 
             aAccount.getSnippetSender(i) + " - " + 
             conv.ConvertToUnicode(aAccount.getSnippetSubject(i));
      
      var msg = conv.ConvertToUnicode(aAccount.getSnippetMsg(i));
      
      gAlerts.addAlert(aAccount.username, subj, msg, aLink);
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {

//var console = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
//console.logStringMessage(aTopic);
//return;

    if (aTopic == "nsPref:changed")
    {
      switch (aData) {
        case "current":
        {
          var current = this._branch.getCharPref("current").split(",");
          var previous = document.getElementById("gm-statusbar-panel").getAttribute("username");
          
          if (current.length > 1 && (current[0] != previous || current[1] != this._current)) {
            this.selectAccount(current[1], false);
          }
          else if (current.length < 2) {
            this._current = -1;
            
            // Toolbar
            if (document.getElementById("gm-toolbar-button")) {
              document.getElementById("gm-toolbar-button").setAttribute("status", "logged-out");
              document.getElementById("gm-toolbar-button").setAttribute("newMail", "false");
              document.getElementById("gm-toolbar-button").setAttribute("unread", "");
              document.getElementById("gm-toolbar-button").setAttribute("username", "");
            }
            
            // Statusbar
            document.getElementById("gm-statusbar-panel").setAttribute("status", "logged-out");
            document.getElementById("gm-statusbar-panel").setAttribute("newMail", "false");
            document.getElementById("gm-statusbar-panel").setAttribute("unread", "");
            document.getElementById("gm-statusbar-panel").setAttribute("username", "");
          }
          
          break;
        }
        case "hide-username":
          this.selectAccount(this._current, false);
          break;
        case "secured-con":
          var secure = this._branch.getBoolPref("secured-con");
          this._scheme = GMAIL_SCHEME[(secure ? 1 : 0)];
          break;
        case "statusbar":
        case "statusbar.always-last":
        case "statusbar.position":
          this._checkStatusbar();
          break;
        case "unread-count-inbox":
        case "unread-count-spam":
        case "unread-count-labels":
          this.selectAccount(this._current, false);
          break;
        case "update-interval":
        {
          var updateInterval = this._branch.getIntPref("update-interval");
          
          for (var i = 0; i < this._manager.getNumAccounts(); i++) {
            var account = this._manager.getAccount(i);
            if (account.loggedIn) account.setTimer(updateInterval);
          }
          break;
        }
      }
    }
    else if (aTopic == "account-login")
    {
      // Get index of the Account
      var index = this._manager.getAccountIndex(aData);
      
      if (index > -1)
        this.selectAccount(index, true);
    }
    else if (aTopic == "reset-unread-count")
    {
      // Toolbar
      if (document.getElementById("gm-toolbar-button")) {
        document.getElementById("gm-toolbar-button").setAttribute("newMail", "false");
        document.getElementById("gm-toolbar-button").setAttribute("unread", "0");
      }
      
      // Statusbar
      document.getElementById("gm-statusbar-panel").setAttribute("newMail", "false");
      document.getElementById("gm-statusbar-panel").setAttribute("unread", "0");
      
      // Resets unread count internally
      this._manager.getAccount(this._current).resetUnreadCount();
    }
    else // Catch all account observers
    {
      // Get account
      var index = this._manager.getAccountIndex(aTopic);
      var account = this._manager.getAccount(index);
      
      // Check for valid account
      if (index == -1 || account == null)
        return;
      
      switch (aData)
      {
        case GMAIL_ACCOUNT_STATE_CONNECTING:
        case GMAIL_ACCOUNT_STATE_ERROR_PASSWORD:
        case GMAIL_ACCOUNT_STATE_ERROR_CAPTCHA:
        case GMAIL_ACCOUNT_STATE_ERROR_NETWORK:
        case GMAIL_ACCOUNT_STATE_ERROR_SERVER:
        case GMAIL_ACCOUNT_STATE_ERROR_TIMEOUT:
          // Do nothing
          break;
        case GMAIL_ACCOUNT_STATE_NO_PASSWORD:
        {
          var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
          var exists = ww.getWindowByName("gm-accounts-" + account.username, null);
          
          if (exists == null) {
            window.openDialog("chrome://gmanager/content/accounts.xul",
                              "gm-accounts-" + account.username,
                              "chrome,dependent=no", "pass", index);
          }
          
          break;
        }
        case GMAIL_ACCOUNT_STATE_LOGGED_OUT:
        {
          try {
            this._observer.removeObserver(this, account.username);
          } catch(e) {}
          
          break;
        }
        case GMAIL_ACCOUNT_STATE_LOGGED_IN:
        {
          var updateInterval = this._branch.getIntPref("update-interval");
          account.setTimer(updateInterval);
          
          break;
        }
        case GMAIL_ACCOUNT_STATE_NEW_MAIL:
        {
          if (this._branch.getBoolPref("new-mail.switch-account"))
            this._current = index;
          
          if ("@mozilla.org/alerts-service;1" in Components.classes &&
              this._branch.getBoolPref("system-notify"))
          {
            var link = this._branch.getBoolPref("system-notify-clickable");
            var subj = account.username + " (Gmail)";
            var unread = account.inboxUnread;
            var msg = this._bundle.getFormattedString("alerts-new-msgs",
                        [unread, this._bundle.getString((unread == 1 ? "msg" : "msgs"))]);
            
            gAlerts.addAlert(account.username, subj, msg, link);
            
            if (this._branch.getBoolPref("system-notify-snippets"))
              this._addAccountSnippets(account, link);
          }
          
          if (this._branch.getBoolPref("sound"))
            gSound.preview(this._branch.getCharPref("sound-loc"));
          
          break;
        }
        case ALERT_NOTIFY_LINK:
          this.loadGmail(0, account);
          break;
      }
      
      if (this._current == index)
        this.selectAccount(this._current, false);
    }
  }
}