// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

var gOverlayCompose = {
  _composeIt: null,
  
  contextMenuShowing: function()
  {
    // Nothing...
  },
  
  contextMenuPopulate: function()
  {
    var bundle = document.getElementById("gm-overlay-bundle");
    
    var popup = getEBI("gm-overlay-context-menu-popup");
    var numAccounts = gOverlay._manager.getNumAccounts();
    var username = gOverlay._branch.getCharPref("compose.mailto.default");
    var item = null;
    
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    item = document.createElement("menuitem");
    item.setAttribute("label", bundle.getString("default-mail-client"));
    item.setAttribute("oncommand", "gOverlayCompose.defaultClick();");
    popup.appendChild(item);
    
    popup.appendChild(document.createElement("menuseparator"));
    
    for (var i = 0; i < numAccounts; i++)
    {
      var account = gOverlay._manager.getAccount(i); 
      var unread = gOverlay._getUnread(account);
      
      item = document.createElement("menuitem");
      item.setAttribute("label", account.username);
      item.setAttribute("default", (account.username == username));
      item.setAttribute("oncommand", "gOverlayCompose.menuClick(event, this.label);");
      
      popup.appendChild(item);
    }
    
    if (numAccounts == 0)
    {
      item = document.createElement("menuitem");
      item.setAttribute("label", bundle.getString("click-add-account"));
      item.setAttribute("oncommand", "window.openDialog('chrome://gmanager/content/accounts.xul', 'accounts', 'chrome,dependent=yes', 'add');");
      popup.appendChild(item);
    }
  },
  
  defaultClick: function()
  {
    var mailtoUrl = "mailto:";
    
    if (this._composeIt != null)
    {
      this._replaceSpaces(false);
      
      mailtoUrl += this._composeIt.to + 
                   "?body=" + this._composeIt.body + 
                   "&subject=" + this._composeIt.su + 
                   "&cc=" + this._composeIt.cc + 
                   "&bcc=" + this._composeIt.bcc;
    }
    
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = ioService.newURI(mailtoUrl, null, null);
    var extProtocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].getService(Components.interfaces.nsIExternalProtocolService);
    
    if (extProtocolSvc)
      extProtocolSvc.loadUrl(uri);
  },
  
  menuClick: function(event, aUsername)
  {
    if (!this._composeIt)
      this._composeIt = this._getComposeObject();
    else
      this._replaceSpaces(true);
    
    gOverlay.loadGmailCompose(aUsername, this._composeIt, false);
  },
  
  linkClick: function(aEvent)
  {
    var linkStr = null;
    
    try {
      // Check for href wrapped with an image or other object
      linkStr = aEvent.originalTarget.parentNode.getAttribute('href').toString();
    } catch(e) {}
    
    try {
      // Check for direct href link
      linkStr = aEvent.originalTarget.getAttribute('href').toString();
    } catch(e) {}
    
    // Checks for a mailto link
    if (linkStr && linkStr.indexOf("mailto:") == 0)
      this._linkify(linkStr);
    else
      this._composeIt = null;
    
    // Checks menu visibility
    this._checkMenuShown();
    
    if (aEvent && (aEvent.button == 0 || aEvent.button == 1) && 
        this._composeIt && gOverlay._branch.getBoolPref("compose.mailto"))
    {
      var username = gOverlay._branch.getCharPref("compose.mailto.default");
      
      this._replaceSpaces(true);
      
      gOverlay.loadGmailCompose(username, this._composeIt, (aEvent.button == 1));
      
      return false;
    }
    
    return window.contentAreaClick(aEvent, false);
  },
  
  _linkify: function(aLinkStr)
  {
    var linkStr = aLinkStr.replace("mailto:", "");
    var split = linkStr.split("?");
    
    this._composeIt = this._getComposeObject();
    this._composeIt.to = split[0];
    
    if (split[1])
    {
      var stuff = split[1].split("&");
      
      for (var i = 0; i < stuff.length; i++)
      {
        var blah = stuff[i].split("=");
        //var code = blah[1].replace(/ /gi, "%2520");
        var code = blah[1];
        
        if (blah[0] == "subject")
          this._composeIt.su = code;
        else if (blah[0] == "body")
          this._composeIt.body = code;
        else if (blah[0] == "cc")
          this._composeIt.cc = code;
        else if (blah[0] == "bcc")
          this._composeIt.bcc = code;
      }
    }
    
    /*
    alert("To = " + this._composeIt.to + "\n" +
          "Subject = " + this._composeIt.su + "\n" +
          "Body = " + this._composeIt.body + "\n" +
          "CC = " + this._composeIt.cc + "\n" +
          "BCC = " + this._composeIt.bcc);
    */
  },
  
  _getComposeObject: function()
  {
    var obj = new Object();
    
    obj.to = "";
    obj.su = "";
    obj.body = "";
    obj.cc = "";
    obj.bcc = "";
    
    return obj;
  },
  
  _replaceSpaces: function(aGmail)
  {
    var r = (aGmail ? "%2520" : "%20");
    
    for (x in this._composeIt)
      this._composeIt[x] = this._composeIt[x].replace(/ /gi, r);
  },
  
  _checkMenuShown: function()
  {
    // Checks if the right click menu is enabled
    var menu = gOverlay._branch.getBoolPref("compose.right-click");
    
    if (menu && gOverlay._branch.getBoolPref("compose.right-click.mailto"))
      this._toggleMenu(this._composeIt == null);
    else
      this._toggleMenu(!menu);
  },
  
  _toggleMenu: function(aShow)
  {
    getEBI("gm-overlay-context-menu").hidden = aShow;
    getEBI("gm-overlay-context-menu").collapsed = aShow;
    
    getEBI("gm-overlay-context-sep").hidden = aShow;
    getEBI("gm-overlay-context-sep").collapsed = aShow;
  }
}