// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

var gOverlayTooltip = {
  _account: null,
  _bundle: null,
  
  populate: function()
  {
    this._bundle = document.getElementById("gm-overlay-bundle");
    
    document.getElementById("gm-toolbar-tooltip-header").hidden = "true";
    document.getElementById("gm-toolbar-tooltip-msg").hidden = "true";
    document.getElementById("gm-toolbar-tooltip-details").hidden = "true";
    
    if (gOverlay._current == -1)
      this._setMsg("click-add-account");
    else
    {
      this._account = gOverlay._manager.getAccount(gOverlay._current);
      
      if (this._account == null) {
        this._setMsg("click-add-account");
        return;
      }
      
      this._setHeader("tb-tooltip-logged-out");
      
      switch (this._account.status)
      {
        case GMAIL_ACCOUNT_STATE_ERROR_PASSWORD:
          this._setMsg("tb-tooltip-msg-error-password");
          break;
        case GMAIL_ACCOUNT_STATE_ERROR_CAPTCHA:
          this._setMsg("tb-tooltip-msg-error-captcha");
          break;
        case GMAIL_ACCOUNT_STATE_ERROR_NETWORK:
          this._setMsg("tb-tooltip-msg-error-network");
          break;
        case GMAIL_ACCOUNT_STATE_ERROR_SERVER:
          this._setMsg("tb-tooltip-msg-error-server");
          break;
        case GMAIL_ACCOUNT_STATE_ERROR_TIMEOUT:
          this._setMsg("tb-tooltip-msg-error-timeout");
          break;
        case GMAIL_ACCOUNT_STATE_CONNECTING:
        {
          this._setHeader("tb-tooltip-connecting");
          
          if (this._account.loggedIn)
            this._setDetails();
          else
            this._setMsg("tb-tooltip-msg-logged-out");
          
          break;
        }
        case GMAIL_ACCOUNT_STATE_NO_PASSWORD:
        case GMAIL_ACCOUNT_STATE_LOGGED_OUT:
          this._setMsg("tb-tooltip-msg-logged-out");
          break;
        case GMAIL_ACCOUNT_STATE_LOGGED_IN:
          this._setHeader("tb-tooltip-logged-in");
          this._setDetails();
          break;
      }
    }
  },
  
  _setHeader: function(aHeader)
  {
    var parent = document.getElementById("gm-toolbar-tooltip-header");
    var item = null;
    
    // Remove all labels
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    item = document.createElement("image");
    item.setAttribute("class", "gm-icon");
    item.setAttribute("iconsize", "small");
    item.setAttribute("status", this._account.status);
    item.setAttribute("newMail", gOverlay._getUnread(this._account) > 0);
    item.setAttribute("style", "margin-right:6px;");
    parent.appendChild(item);
    
    item = document.createElement("label");
    item.setAttribute("value", this._account.username);
    item.setAttribute("style", "font-size: 12px; font-weight: bold; margin: 0px;");
    parent.appendChild(item);
    
    item = document.createElement("label");
    item.setAttribute("value", this._bundle.getString(aHeader));
    item.setAttribute("style", "font-size: 12px; margin: 0 0 0 4px;");
    parent.appendChild(item);
    
    parent.setAttribute("align", "center");
    parent.setAttribute("hidden", "false");
  },
  
  _setMsg: function(aMsg)
  {
    var parent = document.getElementById("gm-toolbar-tooltip-msg");
    var msg = this._bundle.getString(aMsg).split("|");
    var item = null;
    
    // Remove all labels
    while (parent.hasChildNodes())
      parent.removeChild(parent.lastChild);
    
    for (var i = 0; i < msg.length; i++)
    {
      item = document.createElement("label");
      item.setAttribute("value", msg[i]);
      item.setAttribute("style", "margin:2px");
      parent.appendChild(item);
    }
    
    parent.setAttribute("hidden", "false");
  },
  
  _setDetails: function()
  {
    var item = null;
    var unread = null;
    
    var conv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    conv.charset = "UTF-8";
    
    // Inbox
    item = document.getElementById("gm-toolbar-tooltip-details-inbox");
    unread = this._account.inboxUnread;
    item.setAttribute("value", this._bundle.getFormattedString("tb-tooltip-details-inbox", [unread, this._bundle.getString((unread == 1 ? "msg" : "msgs"))]));
    
    // Drafts
    item = document.getElementById("gm-toolbar-tooltip-details-drafts");
    unread = this._account.savedDrafts;
    item.setAttribute("value", this._bundle.getFormattedString("tb-tooltip-details-drafts", [unread, this._bundle.getString((unread == 1 ? "draft" : "drafts"))]));
    item.setAttribute("hidden", (unread == 0));
    
    // Spam
    item = document.getElementById("gm-toolbar-tooltip-details-spam");
    unread = this._account.spamUnread;
    item.setAttribute("value", this._bundle.getFormattedString("tb-tooltip-details-spam", [unread, this._bundle.getString((unread == 1 ? "msg" : "msgs"))]));
    item.setAttribute("hidden", (unread == 0));
    
    // Labels
    var labels = document.getElementById("gm-toolbar-tooltip-details-labels");
    
    while (labels.hasChildNodes())
      labels.removeChild(labels.lastChild);
    
    for (var i = 0; i < this._account.getNumLabels(); i++)
    {
      var label = this._account.getLabel(i).split(",");
      var name = label[0];
      var unread = label[1];
      
      var row = document.createElement("row");
      row.setAttribute("align", "center");
      
      if (unread > 0)
        row.setAttribute("style", "font-weight:bold;");
      
      item = document.createElement("label");
      item.setAttribute("value", conv.ConvertToUnicode(name));
      row.appendChild(item);
      
      item = document.createElement("label");
      item.setAttribute("value", unread);
      item.setAttribute("flex", "1");
      row.appendChild(item);
      
      if (unread > 0 || !gOverlay._branch.getBoolPref("tooltip-labels"))
        labels.appendChild(row);
    }
    
    labels.parentNode.setAttribute("collapsed", !labels.hasChildNodes());
    labels.parentNode.setAttribute("hidden", !labels.hasChildNodes());
    
    // Space
    item = document.getElementById("gm-toolbar-tooltip-details-space");
    item.setAttribute("value", this._bundle.getFormattedString("tb-tooltip-details-space",
                      [this._account.spaceUsed, this._account.percentUsed, this._account.totalSpace]));
    
    // Snippets
    var snippets = document.getElementById("gm-toolbar-tooltip-details-snippets");
    var sep = document.getElementById("gm-toolbar-tooltip-details-snippets-sep");
    
    while (snippets.hasChildNodes())
      snippets.removeChild(snippets.lastChild);
    
    if (gOverlay._branch.getBoolPref("tooltip-snippets"))
    {
      for (var i = 0; i < this._account.getNumSnippets(); i++)
      {
        var time = this._account.getSnippetTime(i);
        var sender = this._account.getSnippetSender(i);
        var subj = this._account.getSnippetSubject(i);
        var msg = this._account.getSnippetMsg(i);
        
        item = document.createElement("label");
        item.setAttribute("value", (i+1) + ". " + time + " - " + conv.ConvertToUnicode(sender) + " - " + conv.ConvertToUnicode(subj));
        item.setAttribute("crop", "end");
        item.setAttribute("style", "margin-top:8px; font-size:10px; font-weight:bold;");
        snippets.appendChild(item);
        
        item = document.createElement("description");
        item.setAttribute("value", conv.ConvertToUnicode(msg));
        item.setAttribute("crop", "end");
        item.setAttribute("style", "font-size:9px;");
        snippets.appendChild(item);
      }
    }
    
    sep.setAttribute("hidden", !snippets.hasChildNodes());
    snippets.setAttribute("hidden", !snippets.hasChildNodes());
    
    var details = document.getElementById("gm-toolbar-tooltip-details");
    details.setAttribute("hidden", "false");
  }
}