// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

function onPrefsLoad() {
  gPrefs.load();
}

function onPrefsUnload() {
  gPrefs._observer.removeObserver(gPrefs, "prefs-account-change");
}

var gPrefs = {
  _manager: null,
  _branch: null,
  _observer: null,
  _username: null,
  _index: -1,
  
  load: function()
  {
    try {
      this._branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(GMANAGER_BRANCH);
      
      this._manager = Components.classes["@longfocus.com/gmanager/manager;1"].getService(Components.interfaces.nsIGmailManager);
      this._manager.load();
      
      this._observer = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      this._observer.addObserver(this, "prefs-account-change", false);
    } catch(e) {}
    
    // Load preferences
    this._loadPrefs();
    
    // Load accounts list
    this._populateAccountsList();
    this._populateDefaultAccountsList();
    
    this._username = this._branch.getCharPref("current").split(",")[0];
    this._index = Math.max(this._manager.getAccountIndex(this._username), 0);
    
    // Selects the current account
    this.selectAccount(this._index);
  },
  
  _loadPrefs: function()
  {
    // Account details
    getEBI("gm-prefs-list").selectedItem = null;
    getEBI("gm-prefs-password-textbox").value = "";
    getEBI("gm-prefs-remem").checked = false;
    
    // General
    getEBI("gm-prefs-auto-login").checked = this._branch.getBoolPref("auto-login");
    getEBI("gm-prefs-secured-con").checked = this._branch.getBoolPref("secured-con");
    getEBI("gm-prefs-statusbar").checked = this._branch.getBoolPref("statusbar");
    getEBI("gm-prefs-statusbar-radiogroup").selectedIndex = (this._branch.getBoolPref("statusbar.always-last") ? 0 : 1);
    getEBI("gm-prefs-statusbar-position").value = this._branch.getIntPref("statusbar.position");
    
    // Compose
    getEBI("gm-prefs-mailto-checkbox").checked = this._branch.getBoolPref("compose.mailto");
    getEBI("gm-prefs-mailto-default-list").selectedItem = document.createElement("menuitem");
    getEBI("gm-prefs-mailto-loc-list").selectedIndex = this._branch.getIntPref("compose.mailto.loc");
    getEBI("gm-prefs-mailto-right-click").checked = this._branch.getBoolPref("compose.right-click");
    getEBI("gm-prefs-mailto-right-click-mailto").checked = this._branch.getBoolPref("compose.right-click.mailto");
    
    // Toolbar
    getEBI("gm-prefs-gmailweb-loc").selectedIndex = this._branch.getIntPref("gmailweb-loc");
    getEBI("gm-prefs-unread-reset").checked = this._branch.getBoolPref("unread-reset");
    getEBI("gm-prefs-hide-username").checked = this._branch.getBoolPref("hide-username");
    getEBI("gm-prefs-tooltip-labels").checked = this._branch.getBoolPref("tooltip-labels");
    getEBI("gm-prefs-tooltip-snippets").checked = this._branch.getBoolPref("tooltip-snippets");
    getEBI("gm-prefs-unread-count-inbox").checked = this._branch.getBoolPref("unread-count-inbox");
    getEBI("gm-prefs-unread-count-spam").checked = this._branch.getBoolPref("unread-count-spam");
    getEBI("gm-prefs-unread-count-labels").checked = this._branch.getBoolPref("unread-count-labels");
    
    // Notifications
    getEBI("gm-prefs-update-interval").value = this._branch.getIntPref("update-interval");
    getEBI("gm-prefs-new-mail-switch-account").checked = this._branch.getBoolPref("new-mail.switch-account");
    getEBI("gm-prefs-system-notify").checked = this._branch.getBoolPref("system-notify");
    getEBI("gm-prefs-system-notify-snippets").checked = this._branch.getBoolPref("system-notify-snippets");
    getEBI("gm-prefs-system-notify-clickable").checked = this._branch.getBoolPref("system-notify-clickable");
    getEBI("gm-prefs-sound").checked = this._branch.getBoolPref("sound");
    getEBI("gm-prefs-sound-loc").value = this._branch.getCharPref("sound-loc");
    
    // Hides the alert notification if not available
    if (!("@mozilla.org/alerts-service;1" in Components.classes))
      getEBI("gm-prefs-alerts").collapsed = true;
  },
  
  _savePrefs: function()
  {
    // General
    this._branch.setBoolPref("auto-login", getEBI("gm-prefs-auto-login").checked);
    this._branch.setBoolPref("secured-con", getEBI("gm-prefs-secured-con").checked);
    this._branch.setBoolPref("statusbar", getEBI("gm-prefs-statusbar").checked);
    this._branch.setBoolPref("statusbar.always-last", getEBI("gm-prefs-statusbar-always-last").selected);
    this._branch.setIntPref("statusbar.position", getEBI("gm-prefs-statusbar-position").value);
    
    // Compose
    this._branch.setBoolPref("compose.mailto", getEBI("gm-prefs-mailto-checkbox").checked);
    this._branch.setCharPref("compose.mailto.default", getEBI("gm-prefs-mailto-default-list").label);
    this._branch.setIntPref("compose.mailto.loc", getEBI("gm-prefs-mailto-loc-list").selectedIndex);
    this._branch.setBoolPref("compose.right-click", getEBI("gm-prefs-mailto-right-click").checked);
    this._branch.setBoolPref("compose.right-click.mailto", getEBI("gm-prefs-mailto-right-click-mailto").checked);
    
    // Toolbar
    this._branch.setIntPref("gmailweb-loc", getEBI("gm-prefs-gmailweb-loc").selectedIndex);
    this._branch.setBoolPref("unread-reset", getEBI("gm-prefs-unread-reset").checked);
    this._branch.setBoolPref("hide-username", getEBI("gm-prefs-hide-username").checked);
    this._branch.setBoolPref("tooltip-labels", getEBI("gm-prefs-tooltip-labels").checked);
    this._branch.setBoolPref("tooltip-snippets", getEBI("gm-prefs-tooltip-snippets").checked);
    this._branch.setBoolPref("unread-count-inbox", getEBI("gm-prefs-unread-count-inbox").checked);
    this._branch.setBoolPref("unread-count-spam", getEBI("gm-prefs-unread-count-spam").checked);
    this._branch.setBoolPref("unread-count-labels", getEBI("gm-prefs-unread-count-labels").checked);
    
    // Notifications
    this._branch.setIntPref("update-interval", getEBI("gm-prefs-update-interval").value);
    this._branch.setBoolPref("new-mail.switch-account", getEBI("gm-prefs-new-mail-switch-account").checked);
    this._branch.setBoolPref("system-notify", getEBI("gm-prefs-system-notify").checked);
    this._branch.setBoolPref("system-notify-snippets", getEBI("gm-prefs-system-notify-snippets").checked);
    this._branch.setBoolPref("system-notify-clickable", getEBI("gm-prefs-system-notify-clickable").checked);
    this._branch.setBoolPref("sound", getEBI("gm-prefs-sound").checked);
    this._branch.setCharPref("sound-loc", getEBI("gm-prefs-sound-loc").value);
  },
  
  _populateAccountsList: function()
  {
    var popup = getEBI("gm-prefs-popup");
    
    // Remove items
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    // Append items
    for (var i = 0; i < this._manager.getNumAccounts(); i++) {
      var account = this._manager.getAccount(i);
      var item = document.createElement("menuitem");
      
      item.setAttribute("id", account.username);
      item.setAttribute("class", "menuitem-non-iconic");
      item.setAttribute("label", account.username);
      item.setAttribute("type", "noapply");
      
      popup.appendChild(item);
    }
  },
  
  _populateDefaultAccountsList: function()
  {
    var popup = getEBI("gm-prefs-mailto-default-popup");
    
    // Remove items
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    // Append items
    for (var i = 0; i < this._manager.getNumAccounts(); i++) {
      var account = this._manager.getAccount(i);
      var item = document.createElement("menuitem");
      
      item.setAttribute("id", account.username);
      item.setAttribute("class", "menuitem-non-iconic");
      item.setAttribute("label", account.username);
      
      popup.appendChild(item);
    }
  },
  
  selectAccount: function(aIndex)
  {
    if (this._manager.getNumAccounts() > 0)
    {
      var account = this._manager.getAccount(aIndex);
      var remem = (this._manager.findPassword(account.username) != "");
      var list = getEBI("gm-prefs-list");
      var popup = getEBI("gm-prefs-popup");
      
      // Selects the account username
      list.selectedItem = popup.childNodes[aIndex];
      
      // Load the account password details
      getEBI("gm-prefs-password-textbox").value = account.password;
      getEBI("gm-prefs-password-textbox").focus();
      getEBI("gm-prefs-remem").checked = remem;
      
      this._username = account.username;
      this._index = aIndex;
      
      // Mailto shizzle
      var defaultMailto = this._branch.getCharPref("compose.mailto.default");
      var defaultIndex = Math.max(this._manager.getAccountIndex(defaultMailto), 0);
      var defaultList = getEBI("gm-prefs-mailto-default-list");
      var defaultPopup = getEBI("gm-prefs-mailto-default-popup");
      
      // Selects the default mailto account username
      defaultList.selectedItem = defaultPopup.childNodes[defaultIndex];
    }
    
    // Check disabled
    this.disabled();
  },
  
  selectSound: function()
  {
    // Opens the sound picker
    var file = gSound.select();
    
    // Checks for file
    if (file)
    {
      // Sets the sound location
      getEBI("gm-prefs-sound-loc").value = file;
      getEBI("gm-prefs-sound-loc").doCommand();
    }
  },
  
  onAction: function(aEvent)
  {
    this.disabled();
    this.validate();
    this.updateApply(aEvent);
  },
  
  disabled: function()
  {
    var disabled = null;
    
    // Accounts
    disabled = (this._manager.getNumAccounts() == 0);
    getEBI("gm-prefs-list").disabled = disabled;
    getEBI("gm-prefs-remove").disabled = disabled;
    getEBI("gm-prefs-password-label").disabled = disabled;
    getEBI("gm-prefs-password-textbox").disabled = disabled;
    getEBI("gm-prefs-remem").disabled = (disabled || getEBI("gm-prefs-password-textbox").value == "");
    
    if (disabled)
    {
      getEBI("gm-prefs-list").selectedItem = null;
      getEBI("gm-prefs-password-textbox").value = "";
    }
    
    // Statusbar
    disabled = !getEBI("gm-prefs-statusbar").checked;
    getEBI("gm-prefs-statusbar-radiogroup").disabled = disabled;
    getEBI("gm-prefs-statusbar-position").disabled = disabled;
    getEBI("gm-prefs-statusbar-position-range").disabled = disabled;
    getEBI("gm-prefs-statusbar-position-range").hidden = disabled;
    
    if (!disabled)
    {
      disabled = !getEBI("gm-prefs-statusbar-specific-position").selected;
      getEBI("gm-prefs-statusbar-position").disabled = disabled;
      getEBI("gm-prefs-statusbar-position-range").value = "0 - " + getStatusbarNumPanels();
      getEBI("gm-prefs-statusbar-position-range").hidden = disabled;
    }
    
    // Mailto
    disabled = !getEBI("gm-prefs-mailto-checkbox").checked;
    getEBI("gm-prefs-mailto-checkbox").disabled = false;
    getEBI("gm-prefs-mailto-default-label").disabled = disabled;
    getEBI("gm-prefs-mailto-default-list").disabled = disabled;
    
    if (this._manager.getNumAccounts() == 0)
    {
      getEBI("gm-prefs-mailto-checkbox").disabled = true;
      getEBI("gm-prefs-mailto-default-label").disabled = true;
      getEBI("gm-prefs-mailto-default-list").disabled = true;
      getEBI("gm-prefs-mailto-default-list").selectedItem = document.createElement("menuitem");
    }
    
    // Right-click menu
    disabled = !getEBI("gm-prefs-mailto-right-click").checked;
    getEBI("gm-prefs-mailto-right-click-mailto").disabled = disabled;
    
    // Alerts
    disabled = !getEBI("gm-prefs-system-notify").checked;
    getEBI("gm-prefs-system-notify-snippets").disabled = disabled;
    getEBI("gm-prefs-system-notify-clickable").disabled = disabled;
    
    // Sounds
    disabled = !getEBI("gm-prefs-sound").checked;
    getEBI("gm-prefs-sound-loc").disabled = disabled;
    getEBI("gm-prefs-sound-browse").disabled = disabled;
    getEBI("gm-prefs-sound-preview").disabled = (getEBI("gm-prefs-sound-loc").value == "" || getEBI("gm-prefs-sound-browse").disabled);
  },
  
  validate: function()
  {
    // Statusbar position
    if (getEBI("gm-prefs-statusbar-specific-position").selected)
    {
      var value = getEBI("gm-prefs-statusbar-position").value;
      var error = (value < 0 || value > getStatusbarNumPanels() || isNaN(value));
      getEBI("gm-prefs-statusbar-position").setAttribute("error", error);
    }
  },
  
  updateApply: function(aEvent)
  {
    if ((aEvent.originalTarget.localName == "button") ||
        (aEvent.originalTarget.localName == "tab") ||
        (aEvent.originalTarget.getAttribute("type") == "noapply")) return;
    
    // Enable Apply button
    getEBI("gm-prefs-apply").disabled = false;
  },
  
  apply: function()
  {
    // Checks if there are any accounts
    if (this._manager.getNumAccounts() > 0)
    {
      var account = this._manager.getAccount(this._index);
      var password = getEBI("gm-prefs-password-textbox").value;
      var store = (getEBI("gm-prefs-password-textbox").value != "" && getEBI("gm-prefs-remem").checked);
      
      this._manager.savePassword(account.username, password, store);
    }
    
    // Save Preferences
    this._savePrefs();
    
    // Disable Apply button
    getEBI("gm-prefs-apply").disabled = true;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "prefs-account-change")
    {
      var num = this._manager.getNumAccounts();
      
      // Load list of accounts
      this._populateAccountsList();
      this._populateDefaultAccountsList();
      
      switch (aData)
      {
        case "add":
          // Selects the new account
          this.selectAccount(num - 1);
          break;
        case "remove":
          var username = this._branch.getCharPref("current").split(",")[0];
          
          if (username == this._username)
            this._branch.setCharPref("current", "");
          
          if (this._index >= num)
            this._index--;
          
          // Selects the appropriate account
          this.selectAccount(this._index);
          break;
      }
    }
  }
}