// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GMANAGER_BRANCH = "gmanager.";
const GMANAGER_WEBSITE = "http://www.longfocus.com/firefox/gmanager/";

const GMAIL_LOGIN = "https://www.google.com/accounts/ServiceLoginAuth";

const ALERT_GMAIL_ICON = "chrome://gmanager/skin/gm-icon.png";
const ALERT_NOTIFY_LINK = "alert-notify-login";

var gAlerts = {
  _alertsService: null,
  _alerts: null,
  _current: null,
  _displaying: false,
  
  addAlert: function(aAccount, aSubject, aSnippet, aLink)
  {
    var exists = false;
    
    if (!this._alerts)
      this._alerts = new Array();
    else
    {
      for (var i = 0; i < this._alerts.length && !exists; i++)
      {
        var alert = this._alerts[i];
        
        if (alert.account == aAccount && alert.subject == aSubject && alert.snippet == aSnippet)
          exists = true;
      }
    }
    
    if (!exists)
    {
      var alert = new Object();
      
      alert.account = aAccount;
      alert.subject = aSubject;
      alert.snippet = aSnippet;
      alert.link = aLink;
      
      this._alerts.push(alert);
      
      if (!this._displaying)
        this.showAlerts();
    }
  },
  
  showAlerts: function()
  {
    if (this._alertsService == null && "@mozilla.org/alerts-service;1" in Components.classes) {
      this._alertsService = Components.classes["@mozilla.org/alerts-service;1"]
                              .getService(Components.interfaces.nsIAlertsService);
    }
    
    if (this._alerts && !this._displaying)
    {
      this._current = this._alerts.shift();
      
      try
      {
        this._alertsService.showAlertNotification(ALERT_GMAIL_ICON,
          this._current.subject, this._current.snippet,
          this._current.link, "", this);
        
        this._displaying = true;
      } catch(e) {}
    }
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic)
    {
      case "alertfinished":
        this.onAlertFinished();
        break;
      case "alertclickcallback":
        this.onAlertClickCallback();
        break;
    }
  },
  
  onAlertFinished: function()
  {
    this._displaying = false;
    
    if (this._alerts.length > 0)
      this.showAlerts();
    else
    {
      this._alerts = null;
      this._current = null;
    }
  },
  
  onAlertClickCallback: function()
  {
    Components.classes["@mozilla.org/observer-service;1"]
      .getService(Components.interfaces.nsIObserverService)
      .notifyObservers(null, this._current.account, ALERT_NOTIFY_LINK);
  }
}

var gSound = {
  _soundService: Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound),
  _ioService: Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService),
  
  select: function()
  {
    var file = null;
    var bundle = document.getElementById("gm-prefs-bundle");
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    
    filePicker.init(window, bundle.getString("prefs-select-sound"), nsIFilePicker.modeOpen);
    filePicker.appendFilter(bundle.getString("prefs-sound-files"), "*.wav");
    filePicker.appendFilters(nsIFilePicker.filterAll);
    
    if (filePicker.show() == nsIFilePicker.returnOK)
      file = filePicker.file.path;
    
    return file;
  },
  
  preview: function(aFile)
  {
    var url = null;
    
    try {
      if (aFile.indexOf("file://") == -1) {
        var localFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        localFile.initWithPath(aFile);
        url = this._ioService.newFileURI(localFile, null, null);
      } else {
        url = this._ioService.newURI(aFile, null, null);
      }
      
      this._soundService.play(url);
    } catch (e) {}
  }
}

var gURI = {
  _observer: Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService),
  _loadMail: false,
  _loadMailStream: null,
  
  load: function(aData, aLocation, aCompose)
  {
    var mimeStream = this._initStream(aData);
    
    switch (aLocation)
    {
      case 0: // Current Tab
        getBrowser().webNavigation.loadURI(GMAIL_LOGIN, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, null, mimeStream, null);
        break;
      case 1: // New Tab (focused)
        getBrowser().selectedTab = getBrowser().addTab();
        getBrowser().webNavigation.loadURI(GMAIL_LOGIN, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, null, mimeStream, null);
        break;
      case 2: // New Tab (in background)
        var newTab = getBrowser().getBrowserForTab(getBrowser().addTab());
        newTab.webNavigation.loadURI(GMAIL_LOGIN, Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE, null, mimeStream, null);
        break;
      case 3: // New Window
        // Adds the window observer
        this._observer.addObserver(this, "browser-window-before-show", false);
        
        this._loadMail = true;
        this._loadMailStream = mimeStream;
        
        //window.open(GMAIL_LOGIN, null, "chrome=no,resizable,width=400,height=500");
        window.open();
        break;
    }
  },
  
  _initStream: function(aData)
  {
    var postData = Components.classes["@mozilla.org/io/string-input-stream;1"].createInstance(Components.interfaces.nsIStringInputStream);
    postData.setData(aData, aData.length);
    
    var mimeStream = Components.classes["@mozilla.org/network/mime-input-stream;1"].createInstance(Components.interfaces.nsIMIMEInputStream);
    mimeStream.addHeader("Content-Type", "application/x-www-form-urlencoded");
    mimeStream.addHeader("Content-Length", aData.length);
    mimeStream.setData(postData);
    
    return mimeStream;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == "browser-window-before-show" && this._loadMail)
    {
      // Removes the window observer
      this._observer.removeObserver(this, "browser-window-before-show");
      
      // Loads Gmail in the new browser window
      getDaBrowser().getBrowser().webNavigation.loadURI(GMAIL_LOGIN,
        Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE,
        null, this._loadMailStream, null);
      
      // Resets the mail values
      this._loadMail = false;
      this._loadMailStream = null;
    }
  }
}

function getEBI(aId) {
  return document.getElementById(aId);
}

function getVersion() {
  var myGUID = "{582195F5-92E7-40a0-A127-DB71295901D7}";
  var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
  var ext = null;
  
  if (em.getItemForID)
    ext = em.getItemForID(myGUID);
   else
    ext = em.getItemList(myGUID, null, {})[0];
  
  return ext.version;
}

function getDaBrowser() {
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  return wm.getMostRecentWindow("navigator:browser");
}

function getStatusbarNumPanels() {
  return getDaBrowser().document.getElementById("status-bar").childNodes.length;
}

function visitSite(aSite) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
  var refString = GMANAGER_WEBSITE + "v/" + getVersion() + "/";
  var refUri = ioService.newURI(refString, null, null);
  
  getDaBrowser().document.getElementById("content").addTab(aSite, refUri);
}